#include "kmc_api/kmc_file.h"
#include "kmc_api/kmer_api.h"
#include <iostream>
#include <fstream>
#include <string>
#include <sstream>

struct ProgramData {
	string input1Name;     // after cutting
	string input2Name;
	string outputFileName;
	unsigned cutoff2;
	bool swapCounters; // true - B's counters will be print as first
};

void usage() {
	std::cerr << "Try: database1Name database2Name outputFileName cutoff2 [swap]" << std::endl;
	exit(EXIT_FAILURE);
}

void parseArgs(int argc, char* argv[], ProgramData& programData) {
	const int minimumArgs = 5;
	if (argc < minimumArgs) {
		std::cerr << "Error: not enough parameters given." << std::endl;
		usage();
	}

	programData.input1Name = argv[1];
	programData.input2Name = argv[2];
	programData.outputFileName = argv[3];

	std::istringstream cutoffStream;

	cutoffStream.str(argv[4]);
	cutoffStream >> programData.cutoff2;

	if (!cutoffStream) {
		std::cerr << "Error: cutoff should be positive integer number." << std::endl;
		usage();
	}

	programData.swapCounters = false;
	if (argc >= minimumArgs + 1) {
		if (argv[minimumArgs] == string("swap")) {
			programData.swapCounters = true;
		}
		else {
			std::cerr << "Error: unknown parameter " << argv[minimumArgs] << "." << std::endl;
			usage();
		}
	}
	if (argc > minimumArgs + 1) {
		std::cerr << "Warning: too many arguments." << std::endl;
	}
}

void generateOutputFile(CKMCFile& database1, CKMCFile& database2, ofstream& outputFile, unsigned kmerLength, const ProgramData& programData) {
	CKmerAPI database1Kmer(kmerLength), database2Kmer(kmerLength);
	float read1Count, read2Count;

	bool noReadBkmer = false; // do not call ReadNextKmer
	while (database1.ReadNextKmer(database1Kmer, read1Count)) {
		while (noReadBkmer || database2.ReadNextKmer(database2Kmer, read2Count)) {
			noReadBkmer = false;
			if (database2Kmer < database1Kmer) {
				continue;
			}
			else if (database2Kmer == database1Kmer) {
				if (read2Count <= programData.cutoff2) {
					outputFile << database1Kmer.to_string() << "\t";
					if (!programData.swapCounters) {
						outputFile << static_cast<unsigned>(read1Count) << "\t" << static_cast<unsigned>(read2Count) << "\n";
					}
					else {
						outputFile << static_cast<unsigned>(read2Count) << "\t" << static_cast<unsigned>(read1Count) << "\n";
					}
				}
				break;
			}
			else {
				outputFile << database1Kmer.to_string() << "\t";
				if (!programData.swapCounters) {
					outputFile << static_cast<unsigned>(read1Count) << "\t" << 0 << "\n";
				}
				else {

					outputFile << 0 << "\t" << static_cast<unsigned>(read1Count) << "\n";
				}
				noReadBkmer = true;
				break;
			}
		}
	}
}

int main(int argc, char* argv[]){
	ProgramData programData;

	parseArgs(argc, argv, programData);

	CKMCFile database1, database2;
	CKMCFileInfo database1Info, database2Info;

	if (!database1.OpenForListing(programData.input1Name)) {
		std::cerr << "Error: cannot open database 1." << std::endl;
		exit(EXIT_FAILURE);
	}

	if (!database2.OpenForListing(programData.input2Name)) {
		std::cerr << "Error: cannot open database B." << std::endl;
		exit(EXIT_FAILURE);
	}

	database1.Info(database1Info);
	database2.Info(database2Info);

	if (database1Info.kmer_length != database2Info.kmer_length) {
		std::cerr << "Error: k-mer lengths of databases are different." << std::endl;
		exit(EXIT_FAILURE);
	}

	ofstream outputFile(programData.outputFileName);
	if (!outputFile) {
		std::cerr << "Error: cannot create output file." << std::endl;
		exit(EXIT_FAILURE);
	}

	generateOutputFile(database1, database2, outputFile, database1Info.kmer_length, programData);
}
